package br.org.usp.ime.corsini.robot.collector;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

import javax.persistence.Query;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

import br.org.usp.ime.corsini.robot.loader.entities.Fund;
import br.org.usp.ime.corsini.robot.persistence.EntityManagerHelper;

/**
 * @author jose.corsini
 *
 */
public class CvmFundInfoLoader {
	
	private static final String CVM_DATE_FORMAT = "yyyy-MM-dd";

	private final Query fundQuery = Fund.getQueryFindByCnpj();
	
	/**
	 * Campos referentes s tags do arquivo de cadastro de fundos
	 * @author jose.corsini
	 *
	 */
	private enum InfoFields
	{
		CNPJ_FUNDO("CNPJ"), NOME("NOME"), START("DT_INICIO"), CATEGORY("CLASSE"), CATEGORY_START("DT_INICIO_CLASSE"), EXCLUSIVE("EXCLUSIVO");

		private String tag;

		private InfoFields(String tag) {
			this.tag = tag;
		}

		private String getTag() {
			return tag;
		}
	}
	
	/**
	 * Trata o arquivo e persiste as informaes de cadastro de fundos
	 * @param completePath
	 */
	public void load(String completePath)
	{
		EntityManagerHelper.startTransaction();
		System.out.println("\r\nProcessando loader de Info...");
		SAXBuilder builder = new SAXBuilder();
		Document doc;
		try {
			doc = builder.build(completePath);
		} catch (JDOMException e) {
			e.printStackTrace();
			return;
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}

		//Percorre os ns do arquivo
		for (Element children : doc.getRootElement().getChildren()) {

			if(children.getName().equals("CABECALHO"))
			{
				//TODO: aproveitar cabecalho
			}else if(children.getName().equals("PARTICIPANTES"))
			{
				//Insere cada um dos registros referentes  um fundo
				for (Element participanteElement : children.getChildren()) {
					insertFund(participanteElement);
				}
			}

		}
		
		EntityManagerHelper.flushAndEndTransaction();
		
		System.out.println("\r\nLoader de Info processado...");
	}

	/**
	 * Insere um determinado fundo e seus dados
	 * @param participanteElement
	 */
	@SuppressWarnings("unchecked")
	private void insertFund(Element participanteElement) {
		
		//Percorre os ns/tags de um fundo
		Fund fund = new Fund();
		for (Element e : participanteElement.getChildren()) {

			if(e.getName().equals(InfoFields.CNPJ_FUNDO.getTag()))
			{
				fund.setCnpj(converteCNPJ(e.getText()));
			}
			else if(e.getName().equals(InfoFields.NOME.getTag()))
			{
				fund.setName(e.getText());
			}
			else if((e.getName().equals(InfoFields.START.getTag())))
			{
				SimpleDateFormat format = new SimpleDateFormat(CVM_DATE_FORMAT);
				try {
					fund.setStart(format.parse(e.getText()));
				} catch (ParseException e1) {
					e1.printStackTrace();
					return;
				}
			}
			else if(e.getName().equals(InfoFields.CATEGORY.getTag()))
			{
				fund.setCategory(e.getText());
			}
			else if((e.getName().equals(InfoFields.CATEGORY_START.getTag())))
			{
				SimpleDateFormat format = new SimpleDateFormat(CVM_DATE_FORMAT);
				try {
					fund.setCategoryStart(format.parse(e.getText()));
				} catch (ParseException e1) {
					e1.printStackTrace();
					return;
				}
			}else if(e.getName().equals(InfoFields.EXCLUSIVE.getTag()))
			{
				fund.setExclusive(trataBoolean(e.getText()));
			}
		}

		//Valida o fundo
		if(fund.validate())
		{
			fundQuery.setParameter("cnpj",fund.getCnpj());
			List<Fund> result = fundQuery.getResultList();
			if(result == null || result.size() == 0)
			{
				EntityManagerHelper.persist(fund);	
			}else
			{
				Fund old = result.get(0);
			
				if(old.equals(fund))
				{
					old.setName(fund.getName());
					old.setStart(fund.getStart());
					old.setCategory(fund.getCategory());
					old.setCategoryStart(fund.getCategoryStart());
					old.setExclusive(fund.getExclusive());
					EntityManagerHelper.merge(old);
				}
			}
		}
		
	}
	
	/**
	 * Converte CNPJ em long
	 * @param cnpj
	 * @return
	 */
	private Long converteCNPJ(String cnpj)
	{
		String cleaned = cnpj.replace(".", "").replace("/", "").replace("-", "");
		return Long.parseLong(cleaned);
	}
	
	/**
	 * Trata valores booleanos
	 * @param text
	 * @return
	 */
	private boolean trataBoolean(String text)
	{
		text = text.toUpperCase();
		if(text.equals("") || text.equals("NAO") || text.equals("NO"))
		{
			return false;
		}
		
		return true;
	}
	

}
