package br.org.usp.ime.corsini.robot.collector;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.persistence.Query;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;

import br.org.usp.ime.corsini.robot.loader.entities.Fund;
import br.org.usp.ime.corsini.robot.loader.entities.FundQuota;
import br.org.usp.ime.corsini.robot.persistence.EntityManagerHelper;

/**
 * @author jose.corsini
 *
 */
public class CvmFundQuotaLoader {

	private static final String CVM_DATE_FORMAT = "yyyy-MM-dd";

	private final Query fundQuery = Fund.getQueryFindByCnpj();

	private final Query fundQuotaQuery = FundQuota.getQueryFindByFundAndDate();

	/**
	 * Campos/tags presentes no arquivo de cotas 
	 */
	private enum CotaFields
	{
		CNPJ_FUNDO("CNPJ_FDO"), DATA_REFERENCIA("DT_COMPTC"), QUOTA("VL_QUOTA"), PL("PATRIM_LIQ");

		private String tag;

		private CotaFields(String tag) {
			this.tag = tag;
		}

		private String getTag() {
			return tag;
		}
	}

	/**
	 * Persiste as informaes dirias dos fundos
	 * @param completePath
	 */
	public void loadDailyInformation(String completePath)
	{

		//Abre o arquivo
		EntityManagerHelper.startTransaction();
		System.out.println("\r\nProcessando Loader de Quota...");
		SAXBuilder builder = new SAXBuilder();
		Document doc;
		try {
			doc = builder.build(completePath);
		} catch (JDOMException e) {
			e.printStackTrace();
			return;
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}

		//Percorre cada elemento/n
		for (Element children : doc.getRootElement().getChildren()) {

			if(children.getName().equals("CABECALHO"))
			{
				//TODO: aproveitar cabecalho
			}else if(children.getName().equals("INFORMES"))
			{
				//Insere cada um dos registros referentes  um fundo
				for (Element informeElement : children.getChildren()) {
					insertQuota(informeElement);
				}
			}

		}

		EntityManagerHelper.flushAndEndTransaction();
		System.out.println("\r\nLoader de Quota processado...");
	}

	/**
	 * Persiste as informaes anuais
	 * @param completePath
	 * @throws IOException
	 */
	public void loadAnnualInformation(String completePath) throws IOException
	{
		EntityManagerHelper.startTransaction();
		System.out.println("\r\nRodando Loader Quota Anual...");
		FileReader reader = new FileReader(completePath);  
		BufferedReader in = new BufferedReader(reader);
		
		
		int i = 0;
		while (in.ready()){  
			insertQuotaLine(in.readLine());
			
			if(i == 3000)
			{
				i = 0;
				EntityManagerHelper.flushAndEndTransaction();
				EntityManagerHelper.startTransaction();
				System.out.println("Outro: " + new SimpleDateFormat("HH:mm:ss").format(new Date()));
			}
			i++;
		}
		
		EntityManagerHelper.flushAndEndTransaction();

		reader.close();  
	}

	/**
	 * Insere uma determinada linha correspondente  cota
	 * @param line
	 */
	@SuppressWarnings("unchecked")
	private void insertQuotaLine(String line) {
		
		String[] values = line.split(";");
		FundQuota fq = new FundQuota();
		
		//Busca o fundo
		Long cnpj = Long.valueOf(values[0]);
		fundQuery.setParameter("cnpj", cnpj);
		List<Fund> result = fundQuery.getResultList();
		if(result == null || result.size() == 0)
		{
			return;
		}

		Fund fund = (Fund) result.get(0);
		fq.setFund(fund);
		
		SimpleDateFormat format = new SimpleDateFormat(CVM_DATE_FORMAT);
		try {
			fq.setDate(format.parse(values[1]));
		} catch (ParseException e1) {
			e1.printStackTrace();
			return;
		}
		fq.setQuota(Double.parseDouble(values[2].replace(",", ".")));
		fq.setNetAsset(Double.parseDouble(values[3].replace(",", ".")));
		
		doIt(fq);
		
	}

	/**
	 * Busca as informaes no elemento refente  cotas
	 * @param informeElement
	 */
	@SuppressWarnings("unchecked")
	private void insertQuota(Element informeElement)
	{
		FundQuota fq = new FundQuota();
		//Percorre os ns correspondentes  uma cota
		for (Element e : informeElement.getChildren()) {

			if(e.getName().equals(CotaFields.CNPJ_FUNDO.getTag()))
			{
				Long cnpj = Long.valueOf(e.getText());
				fundQuery.setParameter("cnpj", cnpj);
				//busca fundo
				List<Fund> result = fundQuery.getResultList();
				if(result == null || result.size() == 0)
				{
					return;
				}

				Fund fund = (Fund) result.get(0);
				fq.setFund(fund);
			}else if(e.getName().equals(CotaFields.DATA_REFERENCIA.getTag()))
			{
				SimpleDateFormat format = new SimpleDateFormat(CVM_DATE_FORMAT);
				try {
					fq.setDate(format.parse(e.getText()));
				} catch (ParseException e1) {
					e1.printStackTrace();
					return;
				}
			}else if(e.getName().equals(CotaFields.QUOTA.getTag()))
			{
				fq.setQuota(Double.parseDouble(e.getText().replace(",", ".")));
			}else if(e.getName().equals(CotaFields.PL.getTag()))
			{
				fq.setNetAsset(Double.parseDouble(e.getText().replace(",", ".")));
			}
		}

		doIt(fq);
	}
	
	/**
	 * Persiste uma cota, ou altera o valor da existente
	 * @param fq
	 */
	@SuppressWarnings("unchecked")
	private void doIt(FundQuota fq)
	{
		if(fq.validate())
		{
			fundQuotaQuery.setParameter("fund", fq.getFund());
			fundQuotaQuery.setParameter("date", fq.getDate());
			List<FundQuota> result = fundQuotaQuery.getResultList();
			if(result == null || result.size() == 0)
			{
				EntityManagerHelper.persist(fq);	
			}else
			{
				FundQuota old = result.get(0);
				if(!old.equals(fq))
				{
					old.setQuota(fq.getQuota());
					old.setNetAsset(fq.getNetAsset());
					EntityManagerHelper.merge(old);
				}
			}
		}
	}
}



